use domo_proto::packet::Packet;

use crate::filter::{Filter, FilterState};

use super::function_filter::FunctionFilter;

pub trait PacketFilter: Filter<Packet> {
    fn filter_packet(&self, filter_packet: Packet) -> FilterState<Packet> {
        self.filter(FilterState::Pass(filter_packet))
    }
}

impl PacketFilter for FunctionFilter {}
impl Filter<Packet> for FunctionFilter {
    fn filter(&self, f: FilterState<Packet>) -> FilterState<Packet> {
        (self.0)(f)
    }
}

impl Filter<Packet> for Packet {
    fn filter(&self, f: FilterState<Packet>) -> FilterState<Packet> {
        match f {
            FilterState::Pass(p) => if &p != self { FilterState::Drop } else { FilterState::Pass(p) },
            FilterState::Skip(p) => if &p != self { FilterState::Drop } else { FilterState::Pass(p) },
            FilterState::Drop => FilterState::Drop,
        }
    }
}