use self::property::Property;
use domo_proto::identifier::Identifier;
use std::collections::HashMap;

pub mod property;

/// Simple Node abstraction.
#[derive(Debug, Clone)]
pub struct Node {
    /// Who owns this node?
    pub parent: Option<Identifier>,
    /// Identifier of node
    pub identifier: Identifier,
    /// Child nodes
    children: HashMap<Identifier, Node>,
    /// All properties of the node.
    pub properties: HashMap<String, Property>,
}

impl Node {
    pub fn new(node_id: Identifier) -> Self {
        Node {
            parent: None,
            identifier: node_id,
            children: HashMap::new(),
            properties: HashMap::new(),
        }
    }

    pub fn add_child(&mut self, node: Node) -> bool {
        if self.children.contains_key(&node.identifier) {
            return false;
        }
        self.children.insert(node.identifier, node);
        true
    }

    pub fn find<'a>(&'a mut self, identifier: &Identifier) -> Option<&'a mut Node> {
        if self.identifier == *identifier {
            return Some(self);
        }
        
        for (_, child) in self.children.iter_mut() {
            if let Some(found) = child.find(identifier) {
                return Some(found);
            }
        }
        
        None
    }
}